from os import getcwd, listdir, sep
from os.path import basename, isdir, isfile, join, normpath, splitext
import sys

class Options(object):
    def __init__(self):
        self.dirs = None

def process_args(argv):
    options = Options()
    if len(argv) == 1:
        options.dirs = ['.']
    else:
        options.dirs = [a for a in argv[1:]]
    return options

def modify_sys_path():
    sys.path.insert(0, getcwd())

def analyse_class(klass):
    return dict([
        (name, value)
        for name, value in klass.__dict__.iteritems()
    ])

def is_a_module(fullname, base):
    return fullname.endswith('.py') and base != '__init__.py'

def get_attr(obj, name):
    if hasattr(obj, name):
        return getattr(obj, name)

def analyse_module(filename):
    # TODO: doesn't work with directory specified as '.'
    modulename = splitext(filename)[0].replace(sep, '.')
    __import__(modulename)
    module = sys.modules[modulename]
    contents = []
    for name, value in module.__dict__.iteritems():
        if (
            not name.startswith('_') and
            hasattr(value, '__module__') and
            getattr(value, '__module__') == modulename
        ):
            contents.append((name, value))
    return contents

def is_a_package(filename):
    return isdir(filename) and isfile(join(filename, '__init__.py'))

def analyse_package(dirname):
    contents = {}
    for filename in listdir(dirname):
        fullname = normpath(join(dirname, filename))
        base = basename(filename)
        if is_a_module(fullname, base):
            contents[base] = analyse_module(fullname)
        if is_a_package(fullname):
            contents[base] = analyse_package(fullname)
    return contents

def display(contents, indent=0):
    if isinstance(contents, dict):
        for name, value in sorted(contents.items()):
            print '%s%s' % ('    ' * indent, name)
            display(value, indent=indent+1)
    elif isinstance(contents, (list, set)):
        for value in sorted(contents):
            display(value, indent=indent)
    else:
        name, value = contents
        if isinstance(value, type):
            print '%s%s' % ('    ' * indent, name)

def main():
    options = process_args(sys.argv)
    modify_sys_path()
    contents = {}
    for dirname in options.dirs:
        contents.update({dirname: analyse_package(dirname)})
    display(contents)

if __name__ == '__main__':
    main()





from os import walk
from os.path import join, normpath



IGNORE_DIRS = ['.svn']
IGNORE_EXTENSIONS = ['.pyc', '.pyo']


def all_files(directory, source=None):
    retval = []
    if source is None:
        source = directory
    for (root, dirs, files) in walk(normpath(source)):
        dirs = filter(lambda d: d not in IGNORE_DIRS, dirs)
        files = filter(
            lambda f: not any(f.endswith(ext) for ext in IGNORE_EXTENSIONS),
            files)
        if files:
            retval.append(
                (directory, [join(root, filename) for filename in files])
            )
    return retval



def get_py2exe_config(name, version, script, console):
    return {
        'console' if console else 'windows': [
            dict(
                script=script,
                # icon_resources=[(1, 'data\%s.ico' % (name,))],
            )
        ],
        'data_files':
            all_files('Microsoft.VC90.CRT', '..\lib\Microsoft.VC90.CRT') + 
            all_files('data'),
        'options':dict(
            py2exe=dict(
                ascii=True, # breaks unicode
                bundle_files=1, # breaks C extensions loaded at runtime
                dist_dir='dist/%s-%s-windows' % (name, version),
                dll_excludes=[
                    # Can we exclude these? Dunno
                    # "pywintypes26.dll",
                    # "pywintypes27.dll",
                ],
                optimize=2,
                excludes=[
                    # silence warnings of missing modules
                    '_imaging_gif',
                    '_scproxy',
                    'clr',
                    'dummy.Process',
                    'email',
                    'email.base64mime',
                    'email.utils',
                    'email.Utils',
                    'ICCProfile',
                    'Image',
                    'IronPythonConsole',
                    'modes.editingmodes',
                    'startup',
                    'System',
                    'System.Windows.Forms.Clipboard',

                    # filter unused .pyd files
                    '_hashlib',
                    '_imaging',
                    '_multiprocessing',
                    '_ssl',
                    '_socket',
                    'bz2',
                    'pyexpat',
                    'pyreadline',
                    'select',
                    'win32api',
                    'win32pipe',

                    # filter unused .pyo files in library.zip
                    'calendar',
                    'cookielib',
                    'difflib',
                    'doctest',
                    'locale',
                    'optparse',
                    'pdb',
                    'pickle',
                    'pyglet.window.xlib',
                    'pyglet.window.carbon',
                    'pyglet.window.carbon.constants',
                    'pyglet.window.carbon.types',
                    'subprocess',
                    'tarfile',
                    'threading',
                    'unittest',
                    'urllib',
                    'urllib2',
                    'win32con',
                    'zipfile',
                ],
            ),
        ),
        'zipfile': None,
    }




from os.path import abspath, dirname, isfile, join
from pprint import pprint
import sys

from .py2exe import get_py2exe_config


def read_description():
    here = abspath(join(dirname(__file__), '..'))

    def first_existing(*possibles):
        for possible in possibles:
            filename = join(here, possible)
            if isfile(filename):
                return filename
        sys.exit("Can't find any of " + ', '.join(possibles))

    filename = first_existing('README', 'README.txt', 'README.rst')

    with open(filename) as fp:
        text = fp.read()

    # return (first_para, rest_of_document)
    paras = text.split('\n\n')
    return paras[0], '\n\n'.join(paras[1:])


def get_sdist_config(name, version, script):
    description, long_description = read_description()
    return dict(
        name=name,
        version=version,
        description=description,
        long_description=long_description,
        keywords='',
        packages=[name, 'setup_utils'],
        scripts=[script],
        #data_files=[('package', ['files'])],
        classifiers=[
            'Development Status :: 1 - Planning',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: BSD License',
            'Operating System :: Microsoft :: Windows',
            'Programming Language :: Python :: 2.7',
        ],    
        # see classifiers http://pypi.python.org/pypi?:action=list_classifiers
    )


def get_config(name, version, script, console=False):
    config = get_sdist_config(name, version, script)
    config.update( get_py2exe_config(name, version, script, console) )
    
    if '--verbose' in sys.argv:
        pprint(config)

    return config




import sys

from setuptools import setup

NAME = 'pyls'
SCRIPT = 'pyls.bat'


def main():
    if 'py2exe' in sys.argv:
        import py2exe
    version = __import__(NAME).__version__
    setup(
        name=NAME,
        version=version,
        entry_points={
            'console_scripts': [
                'pyls=pyls.main.main',
            ],
        },
        author='Jonathan Hartley',
        author_email='tartley@tartley.com',
        url='https://github.com/tartley/pyls',
        license='New BSD License',
    )


if __name__ == '__main__':
    main()

